@@ -8,11 +8,21 @@ from logging import getLogger
 from azure.common import (AzureMissingResourceHttpError, AzureHttpError)
 from azure.storage.blob import BlockBlobService
 from azure.storage.blob.models import ContentSettings
+from azure.storage.common.retry import ExponentialRetry
 
 from .constants import (
     SHA256_DIGEST, ResultStatus, FileHeader, HTTP_HEADER_VALUE_OCTET_STREAM)
 from .encryption_util import (EncryptionMetadata)
 
+from azure.storage.common._http.httpclient import (
+    _HTTPClient,
+    HTTPResponse,
+)
+from azure.storage.common._serialization import _get_data_bytes_or_stream_only
+import requests
+
+logger = getLogger(__name__)
+
 """
 Azure Location: Azure container name + path
 """
@@ -28,6 +38,8 @@ class SnowflakeAzureUtil(object):
     """
     Azure Utility class
     """
+    # max_connections works over this size
+    DATA_SIZE_THRESHOLD = 67108864
 
     @staticmethod
     def create_client(stage_info, use_accelerate_endpoint=False):
@@ -41,8 +53,15 @@ class SnowflakeAzureUtil(object):
         sas_token = stage_credentials[u'AZURE_SAS_TOKEN']
         if sas_token and sas_token.startswith(u'?'):
             sas_token = sas_token[1:]
+        end_point = stage_info['endPoint']
+        if end_point.startswith('blob.'):
+            end_point = end_point[len('blob.'):]
         client = BlockBlobService(account_name=stage_info[u'storageAccount'],
-                                  sas_token=sas_token)
+                                  sas_token=sas_token,
+                                  endpoint_suffix=end_point)
+        client._httpclient = RawBodyReadingClient(session=requests.session(), protocol="https", timeout=2000)
+        client.retry = ExponentialRetry(
+            initial_backoff=1, increment_base=2, max_attempts=60, random_jitter_range=2).retry
         return client
 
     @staticmethod
@@ -70,8 +89,6 @@ class SnowflakeAzureUtil(object):
         :return:  FileHeader if no error,
         u'result_status'] for status.
         """
-
-        logger = getLogger(__name__)
         client = meta[u'client']
         azure_location = SnowflakeAzureUtil.extract_container_name_and_path(
             meta[u'stage_info'][u'location'])
@@ -87,9 +104,11 @@ class SnowflakeAzureUtil(object):
                 encryption_metadata=None
             )
         except AzureHttpError as err:
-            if (
-                    err.status_code == 403 and "Signature not valid in the specified time frame" in str(
-                    err)):
+            logger.debug(u"Caught exception's status code: {status_code} and message: {ex_representation}".format(
+                status_code=err.status_code,
+                ex_representation=str(err)
+            ))
+            if err.status_code == 403 and SnowflakeAzureUtil._detect_azure_token_expire_error(err):
                 logger.debug(u"AZURE Token expired. Renew and retry")
                 meta[u'result_status'] = ResultStatus.RENEW_TOKEN
                 return None
@@ -117,9 +136,16 @@ class SnowflakeAzureUtil(object):
             encryption_metadata=encryption_metadata
         )
 
+    @staticmethod
+    def _detect_azure_token_expire_error(err):
+        if err.status_code != 403:
+            return False
+        errstr = str(err)
+        return "Signature not valid in the specified time frame" in errstr or \
+               "Server failed to authenticate the request." in errstr
+
     @staticmethod
     def upload_file(data_file, meta, encryption_metadata, max_concurrency):
-        logger = getLogger(__name__)
         try:
             azure_metadata = {
                 u'sfcdigest': meta[SHA256_DIGEST],
@@ -149,21 +175,26 @@ class SnowflakeAzureUtil(object):
             path = azure_location.path + meta[u'dst_file_name'].lstrip('/')
 
             client = meta[u'client']
-            callback = meta[u'put_callback'](
-                data_file,
-                os.path.getsize(data_file),
-                output_stream=meta[u'put_callback_output_stream']) if \
-                meta[u'put_callback'] else None
+            callback = None
+            if meta[u'put_azure_callback']:
+                callback = meta[u'put_azure_callback'](
+                    data_file,
+                    os.path.getsize(data_file),
+                    output_stream=meta[u'put_callback_output_stream'],
+                    show_progress_bar=meta[u'show_progress_bar'])
 
             def azure_callback(current, total):
                 callback(current)
+                logger.debug("data transfer progress from sdk callback. "
+                             "current: %s, total: %s",
+                             current, total)
 
             client.create_blob_from_path(
                 azure_location.container_name,
                 path,
                 data_file,
                 progress_callback=azure_callback if
-                meta[u'put_callback'] else None,
+                meta[u'put_azure_callback'] else None,
                 metadata=azure_metadata,
                 max_connections=max_concurrency,
                 content_settings=ContentSettings(
@@ -176,53 +207,114 @@ class SnowflakeAzureUtil(object):
             meta[u'dst_file_size'] = meta[u'upload_size']
             meta[u'result_status'] = ResultStatus.UPLOADED
         except AzureHttpError as err:
-            if (
-                    err.status_code == 403 and "Signature not valid in the specified time frame" in str(
-                    err)):
+            logger.debug(u"Caught exception's status code: {status_code} and message: {ex_representation}".format(
+                status_code=err.status_code,
+                ex_representation=str(err)
+            ))
+            if err.status_code == 403 and SnowflakeAzureUtil._detect_azure_token_expire_error(err):
                 logger.debug(u"AZURE Token expired. Renew and retry")
                 meta[u'result_status'] = ResultStatus.RENEW_TOKEN
                 return None
             else:
                 meta[u'last_error'] = err
                 meta[u'result_status'] = ResultStatus.NEED_RETRY
-        # Comparing with s3, azure haven't experienced OpenSSL.SSL.SysCallError, so we will add logic to catch it only when it happens
+        # Comparing with s3, azure haven't experienced OpenSSL.SSL.SysCallError,
+        # so we will add logic to catch it only when it happens
 
     @staticmethod
     def _native_download_file(meta, full_dst_file_name, max_concurrency):
-        logger = getLogger(__name__)
         try:
             azure_location = SnowflakeAzureUtil.extract_container_name_and_path(
                 meta[u'stage_info'][u'location'])
             path = azure_location.path + meta[u'src_file_name'].lstrip('/')
             client = meta[u'client']
 
-            callback = meta[u'get_callback'](
-                meta[u'src_file_name'],
-                meta[u'src_file_size'],
-                output_stream=meta[u'get_callback_output_stream']) if \
-                meta[u'get_callback'] else None
+            callback = None
+            if meta[u'get_azure_callback']:
+                callback = meta[u'get_azure_callback'](
+                    meta[u'src_file_name'],
+                    meta[u'src_file_size'],
+                    output_stream=meta[u'get_callback_output_stream'],
+                    show_progress_bar=meta[u'show_progress_bar'])
 
             def azure_callback(current, total):
                 callback(current)
+                logger.debug("data transfer progress from sdk callback. "
+                             "current: %s, total: %s",
+                             current, total)
 
             client.get_blob_to_path(
                 azure_location.container_name,
                 path,
                 full_dst_file_name,
                 progress_callback=azure_callback if
-                meta[u'get_callback'] else None,
+                meta[u'get_azure_callback'] else None,
                 max_connections=max_concurrency
             )
 
             meta[u'result_status'] = ResultStatus.DOWNLOADED
         except AzureHttpError as err:
-            if (
-                    err.status_code == 403 and "Signature not valid in the specified time frame" in str(
-                    err)):
+            logger.debug(u"Caught exception's status code: {status_code} and message: {ex_representation}".format(
+                status_code=err.status_code,
+                ex_representation=str(err)
+            ))
+            if err.status_code == 403 and SnowflakeAzureUtil._detect_azure_token_expire_error(err):
                 logger.debug(u"AZURE Token expired. Renew and retry")
                 meta[u'result_status'] = ResultStatus.RENEW_TOKEN
                 return None
             else:
                 meta[u'last_error'] = err
                 meta[u'result_status'] = ResultStatus.NEED_RETRY
-        # Comparing with s3, azure haven't experienced OpenSSL.SSL.SysCallError, so we will add logic to catch it only when it happens
+        # Comparing with s3, azure haven't experienced OpenSSL.SSL.SysCallError,
+        # so we will add logic to catch it only when it happens
+
+
+class RawBodyReadingClient(_HTTPClient):
+    '''
+    This class is needed because the Azure storage Python library has a bug that doesn't
+    allow it to download files with content-encoding=gzip compressed. See
+    https://github.com/Azure/azure-storage-python/issues/509. This client overrides the
+    default HTTP client and downloads uncompressed. This workaround was provided by
+    Microsoft.
+    '''
+    def perform_request(self, request):
+        '''
+        Sends an HTTPRequest to Azure Storage and returns an HTTPResponse. If
+        the response code indicates an error, raise an HTTPError.
+
+        :param HTTPRequest request:
+            The request to serialize and send.
+        :return: An HTTPResponse containing the parsed HTTP response.
+        :rtype: :class:`~azure.storage.common._http.HTTPResponse`
+        '''
+        # Verify the body is in bytes or either a file-like/stream object
+        if request.body:
+            request.body = _get_data_bytes_or_stream_only('request.body', request.body)
+
+        # Construct the URI
+        uri = self.protocol.lower() + '://' + request.host + request.path
+
+        # Send the request
+        response = self.session.request(request.method,
+                                        uri,
+                                        params=request.query,
+                                        headers=request.headers,
+                                        data=request.body or None,
+                                        timeout=self.timeout,
+                                        proxies=self.proxies,
+                                        stream=True)
+
+        # Parse the response
+        status = int(response.status_code)
+        response_headers = {}
+        for key, name in response.headers.items():
+            # Preserve the case of metadata
+            if key.lower().startswith('x-ms-meta-'):
+                response_headers[key] = name
+            else:
+                response_headers[key.lower()] = name
+
+        wrap = HTTPResponse(status, response.reason, response_headers, response.raw.read())
+        response.close()
+
+        return wrap
