@@ -1,23 +1,27 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
 #
-# Copyright (c) 2012-2018 Snowflake Computing Inc. All right reserved.
+# Copyright (c) 2012-2019 Snowflake Computing Inc. All right reserved.
 #
 import decimal
 import os
+import platform
 import sys
 
 from six import string_types, text_type, binary_type, PY2
 
 from snowflake.connector.constants import UTF8
 
+IS_LINUX = platform.system() == 'Linux'
+IS_WINDOWS = platform.system() == 'Windows'
+
 NUM_DATA_TYPES = []
 try:
     import numpy
 
     NUM_DATA_TYPES = [numpy.int8, numpy.int16, numpy.int32, numpy.int64,
                       numpy.float16, numpy.float32, numpy.float64,
-                      numpy.uint8, numpy.uint16, numpy.uint32, numpy.uint64]
+                      numpy.uint8, numpy.uint16, numpy.uint32, numpy.uint64, numpy.bool_]
 except:
     numpy = None
 
@@ -30,10 +34,13 @@ if PY2:
     import httplib
     import Queue
     from HTMLParser import HTMLParser
+    import collections
 
     GET_CWD = os.getcwdu
     BASE_EXCEPTION_CLASS = StandardError  # noqa: F821
     TO_UNICODE = unicode  # noqa: F821
+    ITERATOR = collections.Iterator
+    MAPPING = collections.Mapping
 
     urlsplit = urlparse.urlsplit
     urlunsplit = urlparse.urlunsplit
@@ -45,6 +52,7 @@ if PY2:
     PKCS5_OFFSET = lambda v: ord(v[-1])
     IS_BINARY = lambda v: isinstance(v, bytearray)
 
+    METHOD_NOT_ALLOWED = httplib.METHOD_NOT_ALLOWED
     BAD_GATEWAY = httplib.BAD_GATEWAY
     BAD_REQUEST = httplib.BAD_REQUEST
     REQUEST_TIMEOUT = httplib.REQUEST_TIMEOUT
@@ -53,11 +61,11 @@ if PY2:
     FORBIDDEN = httplib.FORBIDDEN
     UNAUTHORIZED = httplib.UNAUTHORIZED
     INTERNAL_SERVER_ERROR = httplib.INTERNAL_SERVER_ERROR
+    IncompleteRead = httplib.IncompleteRead
     OK = httplib.OK
     BadStatusLine = httplib.BadStatusLine
 
     urlencode = urllib.urlencode
-    proxy_bypass = urllib.proxy_bypass
     unquote = urllib.unquote
     unescape = HTMLParser().unescape
 
@@ -71,10 +79,13 @@ else:
     import urllib.request
     import queue
     import html
+    import collections.abc
 
     GET_CWD = os.getcwd
     BASE_EXCEPTION_CLASS = Exception
     TO_UNICODE = str
+    ITERATOR = collections.abc.Iterator
+    MAPPING = collections.abc.Mapping
 
     urlsplit = urllib.parse.urlsplit
     urlunsplit = urllib.parse.urlunsplit
@@ -86,6 +97,7 @@ else:
     PKCS5_OFFSET = lambda v: v[-1]
     IS_BINARY = lambda v: isinstance(v, (bytes, bytearray))
 
+    METHOD_NOT_ALLOWED = http.client.METHOD_NOT_ALLOWED
     BAD_GATEWAY = http.client.BAD_GATEWAY
     BAD_REQUEST = http.client.BAD_REQUEST
     REQUEST_TIMEOUT = http.client.REQUEST_TIMEOUT
@@ -94,11 +106,11 @@ else:
     FORBIDDEN = http.client.FORBIDDEN
     UNAUTHORIZED = http.client.UNAUTHORIZED
     INTERNAL_SERVER_ERROR = http.client.INTERNAL_SERVER_ERROR
+    IncompleteRead = http.client.IncompleteRead
     OK = http.client.OK
     BadStatusLine = http.client.BadStatusLine
 
     urlencode = urllib.parse.urlencode
-    proxy_bypass = urllib.request.proxy_bypass
     unquote = urllib.parse.unquote
     unescape = html.unescape
 
@@ -110,6 +122,10 @@ IS_STR = lambda v: isinstance(v, STR_DATA_TYPE)
 IS_UNICODE = lambda v: isinstance(v, UNICODE_DATA_TYPE)
 IS_NUMERIC = lambda v: isinstance(v, tuple(NUM_DATA_TYPES))
 
+# Some tests don't need to run on Python34, because SnowSQL specific.
+# SnowSQL runs on Python 3.5+
+PY34_EXACT = sys.version_info[0:2] == (3, 4)
+
 
 def PKCS5_PAD(value, block_size):
     return b"".join(
@@ -142,8 +158,17 @@ def IS_OLD_PYTHON():
            not PY2 and sys.hexversion < 0x03040300
 
 
+try:
+    from inspect import getfullargspec as _get_args
+except ImportError:
+    from inspect import getargspec as _get_args
+
+
+get_args = _get_args
+
+
 """
-Is Python 3.4.3 or 3.5.0
+Is Python 3.5.0
 This is to check if a workaround for http://bugs.python.org/issue23517
 is required or not. 3.6.0 already has the fix.
 No RC or dev version will be checked.
