@@ -144,8 +144,11 @@ PARAMETER_CLIENT_USE_SECURE_STORAGE_FOR_TEMPORARY_CREDENTIAL = \
 PARAMETER_TIMEZONE = u'TIMEZONE'
 PARAMETER_SERVICE_NAME = u'SERVICE_NAME'
 PARAMETER_CLIENT_VALIDATE_DEFAULT_PARAMETERS = u'CLIENT_VALIDATE_DEFAULT_PARAMETERS'
+PARAMETER_PYTHON_CONNECTOR_QUERY_RESULT_FORMAT = u'PYTHON_CONNECTOR_QUERY_RESULT_FORMAT'
 
 HTTP_HEADER_CONTENT_TYPE = u'Content-Type'
+HTTP_HEADER_CONTENT_ENCODING = u'Content-Encoding'
+HTTP_HEADER_ACCEPT_ENCODING = u'Accept-Encoding'
 HTTP_HEADER_ACCEPT = u"accept"
 HTTP_HEADER_USER_AGENT = u"User-Agent"
 HTTP_HEADER_SERVICE_NAME = u'X-Snowflake-Service'
