@@ -15,6 +15,7 @@ from six import u
 from .compat import (BASE_EXCEPTION_CLASS)
 from .constants import (
     FIELD_NAME_TO_ID,
+    PARAMETER_PYTHON_CONNECTOR_QUERY_RESULT_FORMAT
 )
 from .errorcode import (ER_UNSUPPORTED_METHOD,
                         ER_CURSOR_IS_CLOSED,
@@ -37,14 +38,14 @@ logger = getLogger(__name__)
 try:
     import pyarrow
 except ImportError:
-    logger.debug("Failed to import pyarrow. No Apache Arrow result set format can be used.")
+    logger.debug(u"Failed to import pyarrow. Cannot use pandas fetch API")
     pyarrow = None
 
 try:
     from .arrow_result import ArrowResult
     CAN_USE_ARROW_RESULT = True
-except ImportError:
-    logger.debug("Failed to import ArrowResult. No Apache Arrow result set format can be used.")
+except ImportError as e:
+    logger.debug(u"Failed to import ArrowResult. No Apache Arrow result set format can be used. ImportError: %s", e)
     CAN_USE_ARROW_RESULT = False
 
 STATEMENT_TYPE_ID_DML = 0x3000
@@ -81,7 +82,14 @@ class SnowflakeCursor(object):
         u(r'alter\s+session\s+set\s+(.*)=\'?([^\']+)\'?\s*;'),
         flags=re.IGNORECASE | re.MULTILINE | re.DOTALL)
 
-    def __init__(self, connection, json_result_class=JsonResult):
+    def __init__(self, connection, use_dict_result=False, json_result_class=JsonResult):
+        """
+        :param connection: connection created this cursor
+        :param use_dict_result: whether use dict result or not. This variable only applied to
+                                arrow result. When result in json, json_result_class will be
+                                honored
+        :param json_result_class: class that used in json result
+        """
         self._connection = connection
 
         self._errorhandler = Error.default_errorhandler
@@ -105,6 +113,7 @@ class SnowflakeCursor(object):
         self._timezone = None
         self._binary_output_format = None
         self._result = None
+        self._use_dict_result = use_dict_result
         self._json_result_class = json_result_class
 
         self._arraysize = 1  # PEP-0249: defaults to 1
@@ -315,6 +324,19 @@ class SnowflakeCursor(object):
                     u'errno': ER_INVALID_VALUE,
                 })
 
+        # check if current installation include arrow extension or not,
+        # if not, we set statement level query result format to be JSON
+        if not CAN_USE_ARROW_RESULT:
+            logger.debug(u"Cannot use arrow result format, fallback to json format")
+            if statement_params is None:
+                statement_params = {PARAMETER_PYTHON_CONNECTOR_QUERY_RESULT_FORMAT: 'JSON'}
+            else:
+                result_format_val = statement_params.get(PARAMETER_PYTHON_CONNECTOR_QUERY_RESULT_FORMAT)
+                if str(result_format_val).upper() == u'ARROW':
+                    self.check_can_use_arrow_resultset()
+                elif result_format_val is None:
+                    statement_params[PARAMETER_PYTHON_CONNECTOR_QUERY_RESULT_FORMAT] = 'JSON'
+
         self._sequence_counter = self._connection._next_sequence_counter()
         self._request_id = uuid.uuid4()
 
@@ -435,7 +457,7 @@ class SnowflakeCursor(object):
                 _no_results=False,
                 _use_ijson=False,
                 _is_put_get=None,
-                _raise_put_get_error=False,
+                _raise_put_get_error=True,
                 _force_put_overwrite=False):
         u"""
         Executes a command/query
@@ -476,11 +498,7 @@ class SnowflakeCursor(object):
                 processed_params = self._connection._process_params_qmarks(
                     params, self)
         # Skip reporting Key, Value and Type errors
-        except KeyError:
-            raise
-        except ValueError:
-            raise
-        except TypeError:
+        except (KeyError, ValueError, TypeError):
             raise
         except Exception:
             self.connection.incident.report_incident()
@@ -591,6 +609,7 @@ class SnowflakeCursor(object):
     def _init_result_and_meta(self, data, use_ijson=False):
         is_dml = self._is_dml(data)
         self._query_result_format = data.get(u'queryResultFormat', u'json')
+        logger.debug(u"Query result format: %s", self._query_result_format)
 
         if self._total_rowcount == -1 and not is_dml and data.get(u'total') \
                 is not None:
@@ -610,7 +629,7 @@ class SnowflakeCursor(object):
 
         if self._query_result_format == 'arrow':
             self.check_can_use_arrow_resultset()
-            self._result = ArrowResult(data, self)
+            self._result = ArrowResult(data, self, use_dict_result=self._use_dict_result)
         else:
             self._result = self._json_result_class(data, self, use_ijson)
 
@@ -931,4 +950,4 @@ class DictCursor(SnowflakeCursor):
     """
 
     def __init__(self, connection):
-        SnowflakeCursor.__init__(self, connection, DictJsonResult)
+        SnowflakeCursor.__init__(self, connection, use_dict_result=True, json_result_class=DictJsonResult)
