@@ -11,6 +11,10 @@
 """
 OCSP Mode: FAIL_OPEN, FAIL_CLOSED or INSECURE
 """
+import certifi
+from six import wraps
+from urllib3.contrib.pyopenssl import PyOpenSSLContext
+
 from .constants import OCSPMode
 
 FEATURE_OCSP_MODE = OCSPMode.FAIL_OPEN
@@ -37,6 +41,7 @@ import requests.packages.urllib3.util.ssl_ as ssl_
 import requests.packages.urllib3.connection as connection_
 
 from .compat import PY2
+from .compat import get_args
 from .errorcode import (ER_SERVER_CERTIFICATE_REVOKED)
 from .errors import (OperationalError)
 from .ssl_wrap_util import wait_for_read, wait_for_write
@@ -367,14 +372,31 @@ def _verify_callback(cnx, x509, err_no, err_depth, return_code):
     return err_no == 0
 
 
-def ssl_wrap_socket_with_ocsp(
-        sock, keyfile=None, certfile=None, ca_certs=None,
-        ca_cert_dir=None, server_hostname=None, ssl_context=None):
+@wraps(ssl_.ssl_wrap_socket)
+def ssl_wrap_socket_with_ocsp(*args, **kwargs):
+    # Extract host_name
+    hostname_index = get_args(ssl_.ssl_wrap_socket).args.index('server_hostname')
+    server_hostname = args[hostname_index] if len(args) > hostname_index else kwargs.get('server_hostname', None)
+    # Remove context if present
+    ssl_context_index = get_args(ssl_.ssl_wrap_socket).args.index('ssl_context')
+    context_in_args = len(args) > ssl_context_index
+    ssl_context = args[hostname_index] if context_in_args else kwargs.get('ssl_context', None)
+    if not isinstance(ssl_context, PyOpenSSLContext):
+        # Create new default context
+        if context_in_args:
+            new_args = list(args)
+            new_args[ssl_context_index] = None
+            args = tuple(new_args)
+        else:
+            del kwargs['ssl_context']
+    # Fix ca certs location
+    ca_certs_index = get_args(ssl_.ssl_wrap_socket).args.index('ca_certs')
+    ca_certs_in_args = len(args) > ca_certs_index
+    if not ca_certs_in_args and not kwargs.get('ca_certs'):
+        kwargs['ca_certs'] = certifi.where()
+
+    ret = ssl_.ssl_wrap_socket(*args, **kwargs)
 
-    ret = ssl_.ssl_wrap_socket(
-        sock, keyfile=keyfile, certfile=certfile,
-        ca_certs=ca_certs, ca_cert_dir=ca_cert_dir,
-        server_hostname=server_hostname)
     global FEATURE_OCSP_MODE
     global FEATURE_OCSP_RESPONSE_CACHE_FILE_NAME
 
