@@ -1,11 +1,15 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
 #
-# Copyright (c) 2018 Snowflake Computing Inc. All right reserved.
+# Copyright (c) 2018-2019 Snowflake Computing Inc. All right reserved.
 #
 import random
 import time
 
+from logging import getLogger
+
+logger = getLogger(__name__)
+
 try:
     from threading import _Timer as Timer
 except ImportError:
@@ -28,7 +32,10 @@ class HeartBeatTimer(Timer):
         while not self.finished.is_set():
             self.finished.wait(self.interval)
             if not self.finished.is_set():
-                self.function()
+                try:
+                    self.function()
+                except Exception as e:
+                    logger.debug('failed to heartbeat: %s', e)
 
 
 def get_time_millis():
